<?php

namespace wisdmlabs\cpb;

if (! class_exists('WdmCpbInstall')) {
    /**
    * This class create required tables at the time of installation.
    */
    class WdmCpbInstall
    {
        /*
         * Creates all tables required for the plugin. It creates one
         * table in the database. cpb_prefilled_products_data stores the mapping of
         * pre-filled products and their quantity.
         * Gets the charset and collate for the MySQL database.
         * Creates the table of cpb_prefilled_products_data if not already present.
         */
        public static function createTables()
        {
            global $wpdb;
            $wpdb->hide_errors();

            $collate = self::getWpCharsetCollate();

            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

            // Check if fresh installation of plugin
            self::cpbfreshInstalled();

            //Create cpb_prefilled_products_data Table
            self::createPrefilledTable($collate);

            //migration
            self::changeCPBArray();
            self::updateUniqueID();
            self::cpbSettingsPage();
        }

        public static function cpbfreshInstalled()
        {
            global $wpdb;

            if (get_option('wdmcpb_start_version')) {
                return;
            }

            $orderNumber = getLastOrderNumber();


            $cpb_prefilled_products = $wpdb->prefix . 'cpb_prefilled_products_data';
            if (! $wpdb->get_var("SHOW TABLES LIKE '$cpb_prefilled_products';")) {
                update_option('wdmcpb_start_version', CPB_VERSION);
            }

            if ($orderNumber && !is_nan($orderNumber) && get_option('wdmcpb_last_order_updated') != 'no') {
                update_option('wdmcpb_last_order_updated', 'yes');
                update_option('wdmcpb_last_order_id', $orderNumber);
            } else {
                update_option('wdmcpb_last_order_updated', 'no');
            }
        }

        public static function cpbSettingsPage()
        {
            if (!get_option('_grand_total_label')) {
                update_option('_grand_total_label', __('Grand Total', 'brewpixel-wc-custom-product-bundles'));
            }
            if (!get_option('_wdm_enable_giftbox_total')) {
                update_option('_wdm_enable_giftbox_total', 'on');
            }
            if (!get_option('_giftbox_total_label')) {
                update_option('_giftbox_total_label', __('Box Total', 'brewpixel-wc-custom-product-bundles'));
            }
            if (!get_option('_wdm_enable_addbox_total')) {
                update_option('_wdm_enable_addbox_total', 'on');
            }
            if (!get_option('_addbox_total_label')) {
                update_option('_addbox_total_label', __('Box Charges', 'brewpixel-wc-custom-product-bundles'));
            }
            if (!get_option('_wdmcpb_anonymize_mgs')) {
                update_option('_wdmcpb_anonymize_mgs', 'on');
            }
        }

        public static function createPrefilledTable($collate)
        {
            global $wpdb;

            $cpb_prefilled_products = $wpdb->prefix . 'cpb_prefilled_products_data';            
            if (! $wpdb->get_var("SHOW TABLES LIKE '$cpb_prefilled_products';")) {
                $prefilled_table_query  = "
                CREATE TABLE IF NOT EXISTS {$cpb_prefilled_products} (
                                    id bigint(20) PRIMARY KEY NOT NULL AUTO_INCREMENT,
                                    cpb_product_id bigint(20),
                                    unique_prod_id varchar(35),
                                    prefill_product_id bigint(20),
                                    prefill_quantity bigint(20),
                                    prefill_mandatory TINYINT(1),
                                    product_type varchar(35),
                                    INDEX product_id (cpb_product_id),
                                    INDEX unique_id (unique_prod_id),
                                    INDEX user_id (prefill_product_id)
                                ) $collate;
                                ";
                @dbDelta($prefilled_table_query);
            }

            self::alterTable();
        }

        public static function alterTable()
        {
            global $wpdb;
            $cpb_prefilled_products = $wpdb->prefix . 'cpb_prefilled_products_data';
            if (!$wpdb->get_var("SHOW COLUMNS FROM `$cpb_prefilled_products` LIKE 'unique_prod_id';")) {
                $wpdb->query("ALTER TABLE $cpb_prefilled_products ADD unique_prod_id varchar(35) NOT NULL DEFAULT '', ADD INDEX unique_id (unique_prod_id)");

            }

            if (!$wpdb->get_var("SHOW COLUMNS FROM `$cpb_prefilled_products` LIKE 'product_type';")) {
                $wpdb->query("ALTER TABLE $cpb_prefilled_products ADD product_type varchar(35) NOT NULL DEFAULT 'simple'");

            }
        }

        public static function changeCPBArray()
        {
            if (!get_option('cpb_array_changed')) {
                // generating default unique id for already existing records
                $cpbIds = getCPBProductIds();

                if (self::updateNewArrays($cpbIds)) {
                    update_option('cpb_array_changed', true);
                }
            }
        }

        public static function updateUniqueID()
        {
            if (!get_option('cpb_unique_id_updated')) {
                global $wpdb;
                $cpb_prefilled_products = $wpdb->prefix . 'cpb_prefilled_products_data';
                $prefillIds = $wpdb->get_col("SELECT prefill_product_id FROM {$cpb_prefilled_products}");
                // $updatedPrefillIds = array();

                foreach ($prefillIds as $key => $id) {
                    $status = $wpdb->update(
                        $cpb_prefilled_products,
                        array(
                            "unique_prod_id" => md5($id)
                        ),
                        array(
                            "prefill_product_id" => $id
                        ),
                        array(
                            '%s'
                        ),
                        array(
                            '%d'
                        )
                    );
                }

                update_option('cpb_unique_id_updated', true);
            }
        }

        public static function updateNewArrays($cpbIds)
        {
            if (empty($cpbIds)) {
                return false;
            }

            $updatedArray = array();

            foreach ($cpbIds as $productID) {
                $cpbArray = maybe_unserialize(
                    get_post_meta(
                        $productID,
                        '_bundle_data',
                        true
                    )
                );

                self::updateArray($cpbArray, $productID);
            }
            return true;
        }

        public static function updateArray($cpbArray, $productID)
        {
            if (empty($cpbArray)) {
                return;
            }
            $updatedArray = array();

            foreach ($cpbArray as $bundleId => $value) {
                if (getType($bundleId) == 'string') {
                    update_option('cpb_array_changed', true);
                    return;
                }
                
                $product = wc_get_product($bundleId);
                $max_val =  0;
                $bundle_item_quantity = get_post_meta($bundleId, '_stock', true);
                if (!empty($bundle_item_quantity)) {
                    $max_val = $bundle_item_quantity;
                }
                $updatedArray = JsonProductSearch::getSimpleProduct($bundleId, $product, $updatedArray);
                $updatedArray[md5($bundleId)]['id'] = md5($bundleId);
                $updatedArray[md5($bundleId)]['text'] = $updatedArray[md5($bundleId)]['text_name'];
                unset($updatedArray[md5($bundleId)]['text_name']);
            }
            update_post_meta($productID, '_bundle_data', $updatedArray);
        }

        /**
        * Gets the default charset and collate for the MySQL database.
        * @return string $charset_collate charset and collate for the MySQL 
        * database.
        */
        protected static function getWpCharsetCollate()
        {
            global $wpdb;
            $charset_collate = '';

            if (! empty($wpdb->charset)) {
                $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
            }

            if (! empty($wpdb->collate)) {
                $charset_collate .= " COLLATE $wpdb->collate";
            }

            return $charset_collate;
        }

    }
}
