<?php

/**
 * CPB Add-on Product
 * @version 3.3.0
 */

// Exit if accessed directly
if (! defined('ABSPATH')) {
    exit;
}

global $woocommerce, $post;
//Get the add-on product details to display
$wdm_manage_stock = get_option('woocommerce_manage_stock');
$cpb_hide_stock = get_option('_wdm_hide_stock');

$bundled_item_id = $item_data['product_type'] == "variation" ? $item_data['variation_id'] : $item_data['product_id'];
$id = $item_data['id'];

$bundled_product = wc_get_product($bundled_item_id);
$product_bundle = wc_get_product( $post->ID );
$bundle_sku = $product_bundle->get_sku();
$factor1 = $bundled_product->get_attribute( 'pa_factor1' );
$factor2 = $bundled_product->get_attribute( 'pa_factor2' );
$factor3 = $bundled_product->get_attribute( 'pa_factor3' );
$factor4 = $bundled_product->get_attribute( 'pa_factor4' );

switch ( $bundle_sku ) {
	case 'PKG BOX XXXPK PRP S3':
		$bundled_product_quantity = $factor1;
		break;
	case 'PKG BOX XXXPK PRP S5':
		$bundled_product_quantity = $factor2;
		break;
	case 'PKG BOX XXXPK FRT S3':
		$bundled_product_quantity = $factor3;
		break;
	case 'PKG BOX XXXPK FRT L3':
		$bundled_product_quantity = $factor4;
		break;
	default:
		$bundled_product_quantity = 1;
		break;
}

$price = wc_get_price_to_display($bundled_product) * (int)$bundled_product_quantity;

$wdm_item_product = new stdClass();

if (get_post_type($bundled_item_id) == 'product_variation') {
    $wdm_item_product = new \WC_Product_Variation($bundled_item_id);
} else {
    $wdm_item_product = new \WC_Product($bundled_item_id);
}

$wdmItemPriceHtml = $wdm_item_product->get_price_html();
$isSoldIndividually = ($wdm_item_product->is_sold_individually() == true) ? 1 : 0;

$classes = apply_filters(
    'wdm_cpb_addon_product_div_classes',
    array(
        'mobile_list_layout',
        'product',
        'bundled_product',
        'bundled_product_summary',
        $stckMessage['wdm_no_stock'],
        $stckMessage['backorderClass']
    ),
    $post
);

if ($canBeSwapped !== false) {
    $classes[] = 'unpurchasable-product';
}
// Not allowing this through filter becasuse developers should not change this 
// class name. This class is
// necessary to identify exact product code needs to work with.
$classes[] =  "mobile_bundled_product_{$id}";

$classes = implode(" ", $classes);
//Display the add-on products on basis of the stock status and settings ,
//whether they should be displayed or not.
?>

<div
    class = "<?php echo $classes; ?>"
    data-product-id = "<?php echo $item_data['id']; ?>"
    data-product-price = "<?php echo $price; ?>"
    data-sold-individually = "<?php echo $isSoldIndividually; ?>"
>
    <?php
    //image template
    do_action('wdm_add_on_product_image', $item_data, $wdm_item_product);
?>
    <div class='mobile-list-layout-addon-product-info'>
        <?php
        //title template
        do_action('wdm_add_on_product_title', $item_data['display_text'], $item_data);
        $product_bundle = wc_get_product( $post->ID );
        $bundle_sku = $product_bundle->get_sku();
        if ( 'PKG BOX XXXPK H-LRG' === $bundle_sku ) {
	        foreach ( wc_get_product_terms( $item_data['product_id'], 'pa_per-portion', array( 'fields' => 'names' ) ) as $attribute_value ) {
		        $measurement     = wc_get_product_terms( $item_data['product_id'], 'pa_uom', array( 'fields' => 'names' ) );
		        $piece_or_pieces = (int) $attribute_value > 1 ? array( ' pieces' ) : array( ' piece' );
		        $measurement     = empty( $measurement ) ? $piece_or_pieces : $measurement;
		        // Outputting the attribute values one by one
		        echo '<span class="portion">Portion is ' . (int) $attribute_value . $measurement[0] . '</span>';
	        }
	        if ( $cpb_hide_stock == 'off' && ( $wdm_manage_stock == 'yes' || ! $wdm_item_product->is_in_stock() || ( $wdm_item_product->is_in_stock() && $wdm_item_product->is_sold_individually() ) ) ) {
		        echo $stckMessage['price_message'];
	        }

	        if ( $per_product_pricing ) {
		        ?>
                <p class="wdm_price">
			        <?php echo $wdmItemPriceHtml; ?>
                </p>
		        <?php
	        }
        }

        $text = str_replace('_', '', $bundled_item_id);
        ?>
    </div>

    <div class='mobile-list-layout-add-on-product-quantity'>
        <div
            class = "cart"
            data-original-id = "<?php echo $bundled_item_id; ?>"
            data-bundled-item-id = "<?php echo $item_data['id']; ?>"
            data-product-id = "<?php echo $post->ID.$text; ?>"
            data-bundle-id = "<?php echo $post->ID; ?>"
            data-bundle-sku = "<?php echo $bundle_sku; ?>"
        >

            <?php
                do_action('wdm_add_on_product_quantity', $item_data);
            ?>
        </div>
    </div>
    <div class="cpb-clear"></div>
</div>
